@@ -7,6 +7,7 @@ import hashlib
 import pathlib
 import sqlite_utils
 from sqlite_utils.db import AlterError
+import textwrap
 import itertools
 import json
 import os
@@ -274,7 +275,7 @@ def optimize(path, tables, no_vacuum, load_extension):
 @click.argument("tables", nargs=-1)
 @load_extension_option
 def rebuild_fts(path, tables, load_extension):
-    """Rebuild specific FTS tables, or all FTS tables if none are specified"""
+    """Rebuild all or specific FTS tables"""
     db = sqlite_utils.Database(path)
     _load_extensions(db, load_extension)
     if not tables:
@@ -547,6 +548,43 @@ def disable_wal(path, load_extension):
         db.disable_wal()
 
 
+@cli.command(name="enable-counts")
+@click.argument(
+    "path",
+    type=click.Path(exists=True, file_okay=True, dir_okay=False, allow_dash=False),
+    required=True,
+)
+@click.argument("tables", nargs=-1)
+@load_extension_option
+def enable_counts(path, tables, load_extension):
+    "Configure triggers to update a _counts table with row counts"
+    db = sqlite_utils.Database(path)
+    _load_extensions(db, load_extension)
+    if not tables:
+        db.enable_counts()
+    else:
+        # Check all tables exist
+        bad_tables = [table for table in tables if not db[table].exists()]
+        if bad_tables:
+            raise click.ClickException("Invalid tables: {}".format(bad_tables))
+        for table in tables:
+            db[table].enable_counts()
+
+
+@cli.command(name="reset-counts")
+@click.argument(
+    "path",
+    type=click.Path(exists=True, file_okay=True, dir_okay=False, allow_dash=False),
+    required=True,
+)
+@load_extension_option
+def reset_counts(path, load_extension):
+    "Reset calculated counts in the _counts table"
+    db = sqlite_utils.Database(path)
+    _load_extensions(db, load_extension)
+    db.reset_counts()
+
+
 def insert_upsert_options(fn):
     for decorator in reversed(
         (
@@ -630,12 +668,19 @@ def insert_upsert_implementation(
             reader = csv_std.reader(json_file, dialect=dialect)
             headers = next(reader)
             docs = (dict(zip(headers, row)) for row in reader)
-    elif nl:
-        docs = (json.loads(line) for line in json_file)
     else:
-        docs = json.load(json_file)
-        if isinstance(docs, dict):
-            docs = [docs]
+        try:
+            if nl:
+                docs = (json.loads(line) for line in json_file)
+            else:
+                docs = json.load(json_file)
+                if isinstance(docs, dict):
+                    docs = [docs]
+        except json.decoder.JSONDecodeError:
+            raise click.ClickException(
+                "Invalid JSON - use --csv for CSV or --tsv for TSV files"
+            )
+
     extra_kwargs = {"ignore": ignore, "replace": replace, "truncate": truncate}
     if not_null:
         extra_kwargs["not_null"] = set(not_null)
@@ -1099,6 +1144,53 @@ def rows(
     )
 
 
+@cli.command()
+@click.argument(
+    "path",
+    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
+    required=True,
+)
+@click.argument("tables", nargs=-1)
+@output_options
+@load_extension_option
+@click.pass_context
+def triggers(
+    ctx,
+    path,
+    tables,
+    nl,
+    arrays,
+    csv,
+    tsv,
+    no_headers,
+    table,
+    fmt,
+    json_cols,
+    load_extension,
+):
+    "Show triggers configured in this database"
+    sql = "select name, tbl_name as [table], sql from sqlite_master where type = 'trigger'"
+    if tables:
+        quote = sqlite_utils.Database(memory=True).quote
+        sql += " and [table] in ({})".format(
+            ", ".join(quote(table) for table in tables)
+        )
+    ctx.invoke(
+        query,
+        path=path,
+        sql=sql,
+        nl=nl,
+        arrays=arrays,
+        csv=csv,
+        tsv=tsv,
+        no_headers=no_headers,
+        table=table,
+        fmt=fmt,
+        json_cols=json_cols,
+        load_extension=load_extension,
+    )
+
+
 @cli.command()
 @click.argument(
     "path",
@@ -1354,6 +1446,93 @@ def insert_files(
             )
 
 
+@cli.command(name="analyze-tables")
+@click.argument(
+    "path",
+    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False, exists=True),
+    required=True,
+)
+@click.argument("tables", nargs=-1)
+@click.option(
+    "-c",
+    "--column",
+    "columns",
+    type=str,
+    multiple=True,
+    help="Specific columns to analyze",
+)
+@click.option("--save", is_flag=True, help="Save results to _analyze_tables table")
+@load_extension_option
+def analyze_tables(
+    path,
+    tables,
+    columns,
+    save,
+    load_extension,
+):
+    "Analyze the columns in one or more tables"
+    db = sqlite_utils.Database(path)
+    _load_extensions(db, load_extension)
+    if not tables:
+        tables = db.table_names()
+    todo = []
+    table_counts = {}
+    for table in tables:
+        table_counts[table] = db[table].count
+        for column in db[table].columns:
+            if not columns or column.name in columns:
+                todo.append((table, column.name))
+    # Now we now how many we need to do
+    for i, (table, column) in enumerate(todo):
+        column_details = db[table].analyze_column(
+            column, total_rows=table_counts[table], value_truncate=80
+        )
+        if save:
+            db["_analyze_tables_"].insert(
+                column_details._asdict(), pk=("table", "column"), replace=True
+            )
+        most_common_rendered = _render_common(
+            "\n\n  Most common:", column_details.most_common
+        )
+        least_common_rendered = _render_common(
+            "\n\n  Least common:", column_details.least_common
+        )
+        details = (
+            (
+                textwrap.dedent(
+                    """
+        {table}.{column}: ({i}/{total})
+
+          Total rows: {total_rows}
+          Null rows: {num_null}
+          Blank rows: {num_blank}
+
+          Distinct values: {num_distinct}{most_common_rendered}{least_common_rendered}
+        """
+                )
+                .strip()
+                .format(
+                    i=i + 1,
+                    total=len(todo),
+                    most_common_rendered=most_common_rendered,
+                    least_common_rendered=least_common_rendered,
+                    **column_details._asdict()
+                )
+            )
+            + "\n"
+        )
+        click.echo(details)
+
+
+def _render_common(title, values):
+    if values is None:
+        return ""
+    lines = [title]
+    for value, count in values:
+        lines.append("    {}: {}".format(count, value))
+    return "\n".join(lines)
+
+
 FILE_COLUMNS = {
     "name": lambda p: p.name,
     "path": lambda p: str(p),
