@@ -20,3 +20,8 @@ def documented_commands():
 @pytest.mark.parametrize("command", cli.cli.commands.keys())
 def test_commands_are_documented(documented_commands, command):
     assert command in documented_commands
+
+
+@pytest.mark.parametrize("command", cli.cli.commands.values())
+def test_commands_have_docstrings(command):
+    assert command.__doc__, "{} is missing a docstring".format(command)
