@@ -29,9 +29,23 @@ def test_enable_fts(fresh_db):
         "searchable_fts_docsize",
         "searchable_fts_stat",
     ] == fresh_db.table_names()
-    assert [("tanuki are running tricksters", "Japan", "foo")] == table.search("tanuki")
-    assert [("racoons are biting trash pandas", "USA", "bar")] == table.search("usa")
-    assert [] == table.search("bar")
+    assert [
+        {
+            "rowid": 1,
+            "text": "tanuki are running tricksters",
+            "country": "Japan",
+            "not_searchable": "foo",
+        }
+    ] == list(table.search("tanuki"))
+    assert [
+        {
+            "rowid": 2,
+            "text": "racoons are biting trash pandas",
+            "country": "USA",
+            "not_searchable": "bar",
+        }
+    ] == list(table.search("usa"))
+    assert [] == list(table.search("bar"))
 
 
 def test_enable_fts_escape_table_names(fresh_db):
@@ -49,9 +63,23 @@ def test_enable_fts_escape_table_names(fresh_db):
         "http://example.com_fts_docsize",
         "http://example.com_fts_stat",
     ] == fresh_db.table_names()
-    assert [("tanuki are running tricksters", "Japan", "foo")] == table.search("tanuki")
-    assert [("racoons are biting trash pandas", "USA", "bar")] == table.search("usa")
-    assert [] == table.search("bar")
+    assert [
+        {
+            "rowid": 1,
+            "text": "tanuki are running tricksters",
+            "country": "Japan",
+            "not_searchable": "foo",
+        }
+    ] == list(table.search("tanuki"))
+    assert [
+        {
+            "rowid": 2,
+            "text": "racoons are biting trash pandas",
+            "country": "USA",
+            "not_searchable": "bar",
+        }
+    ] == list(table.search("usa"))
+    assert [] == list(table.search("bar"))
 
 
 def test_enable_fts_table_names_containing_spaces(fresh_db):
@@ -72,12 +100,20 @@ def test_populate_fts(fresh_db):
     table = fresh_db["populatable"]
     table.insert(search_records[0])
     table.enable_fts(["text", "country"], fts_version="FTS4")
-    assert [] == table.search("trash pandas")
+    assert [] == list(table.search("trash pandas"))
     table.insert(search_records[1])
-    assert [] == table.search("trash pandas")
+    assert [] == list(table.search("trash pandas"))
     # Now run populate_fts to make this record available
     table.populate_fts(["text", "country"])
-    assert [("racoons are biting trash pandas", "USA", "bar")] == table.search("usa")
+    rows = list(table.search("usa"))
+    assert [
+        {
+            "rowid": 2,
+            "text": "racoons are biting trash pandas",
+            "country": "USA",
+            "not_searchable": "bar",
+        }
+    ] == rows
 
 
 def test_populate_fts_escape_table_names(fresh_db):
@@ -85,12 +121,19 @@ def test_populate_fts_escape_table_names(fresh_db):
     table = fresh_db["http://example.com"]
     table.insert(search_records[0])
     table.enable_fts(["text", "country"], fts_version="FTS4")
-    assert [] == table.search("trash pandas")
+    assert [] == list(table.search("trash pandas"))
     table.insert(search_records[1])
-    assert [] == table.search("trash pandas")
+    assert [] == list(table.search("trash pandas"))
     # Now run populate_fts to make this record available
     table.populate_fts(["text", "country"])
-    assert [("racoons are biting trash pandas", "USA", "bar")] == table.search("usa")
+    assert [
+        {
+            "rowid": 2,
+            "text": "racoons are biting trash pandas",
+            "country": "USA",
+            "not_searchable": "bar",
+        }
+    ] == list(table.search("usa"))
 
 
 def test_fts_tokenize(fresh_db):
@@ -103,7 +146,7 @@ def test_fts_tokenize(fresh_db):
             ["text", "country"],
             fts_version="FTS{}".format(fts_version),
         )
-        assert [] == table.search("bite")
+        assert [] == list(table.search("bite"))
         # Test WITH stemming
         table.disable_fts()
         table.enable_fts(
@@ -111,9 +154,14 @@ def test_fts_tokenize(fresh_db):
             fts_version="FTS{}".format(fts_version),
             tokenize="porter",
         )
-        assert [("racoons are biting trash pandas", "USA", "bar")] == table.search(
-            "bite"
-        )
+        rows = list(table.search("bite", order_by="rowid"))
+        assert len(rows) == 1
+        assert {
+            "rowid": 2,
+            "text": "racoons are biting trash pandas",
+            "country": "USA",
+            "not_searchable": "bar",
+        }.items() <= rows[0].items()
 
 
 def test_optimize_fts(fresh_db):
@@ -132,15 +180,32 @@ def test_optimize_fts(fresh_db):
         fresh_db[table_name].optimize()
 
 
-def test_enable_fts_w_triggers(fresh_db):
+def test_enable_fts_with_triggers(fresh_db):
     table = fresh_db["searchable"]
     table.insert(search_records[0])
     table.enable_fts(["text", "country"], fts_version="FTS4", create_triggers=True)
-    assert [("tanuki are running tricksters", "Japan", "foo")] == table.search("tanuki")
+    rows1 = list(table.search("tanuki"))
+    assert len(rows1) == 1
+    assert rows1 == [
+        {
+            "rowid": 1,
+            "text": "tanuki are running tricksters",
+            "country": "Japan",
+            "not_searchable": "foo",
+        }
+    ]
     table.insert(search_records[1])
     # Triggers will auto-populate FTS virtual table, not need to call populate_fts()
-    assert [("racoons are biting trash pandas", "USA", "bar")] == table.search("usa")
-    assert [] == table.search("bar")
+    rows2 = list(table.search("usa"))
+    assert rows2 == [
+        {
+            "rowid": 2,
+            "text": "racoons are biting trash pandas",
+            "country": "USA",
+            "not_searchable": "bar",
+        }
+    ]
+    assert [] == list(table.search("bar"))
 
 
 @pytest.mark.parametrize("create_triggers", [True, False])
@@ -183,15 +248,29 @@ def test_rebuild_fts(fresh_db, table_to_fix):
     table.insert(search_records[0])
     table.enable_fts(["text", "country"])
     # Run a search
-    assert [("tanuki are running tricksters", "Japan", "foo")] == table.search("tanuki")
+    rows = list(table.search("tanuki"))
+    assert len(rows) == 1
+    assert {
+        "rowid": 1,
+        "text": "tanuki are running tricksters",
+        "country": "Japan",
+        "not_searchable": "foo",
+    }.items() <= rows[0].items()
     # Delete from searchable_fts_data
     fresh_db["searchable_fts_data"].delete_where()
     # This should have broken the index
     with pytest.raises(sqlite3.DatabaseError):
-        table.search("tanuki")
+        list(table.search("tanuki"))
     # Running rebuild_fts() should fix it
     fresh_db[table_to_fix].rebuild_fts()
-    assert [("tanuki are running tricksters", "Japan", "foo")] == table.search("tanuki")
+    rows2 = list(table.search("tanuki"))
+    assert len(rows2) == 1
+    assert {
+        "rowid": 1,
+        "text": "tanuki are running tricksters",
+        "country": "Japan",
+        "not_searchable": "foo",
+    }.items() <= rows2[0].items()
 
 
 @pytest.mark.parametrize("invalid_table", ["does_not_exist", "not_searchable"])
@@ -276,3 +355,85 @@ def test_enable_fts_replace_does_nothing_if_args_the_same():
     db["books"].enable_fts(["title", "author"], create_triggers=True, replace=True)
     # The only SQL that executed should be select statements
     assert all(q[0].startswith("select ") for q in queries)
+
+
+@pytest.mark.parametrize(
+    "kwargs,fts,expected",
+    [
+        (
+            {},
+            "FTS5",
+            (
+                "with original as (\n"
+                "    select\n"
+                "        rowid,\n"
+                "        *\n"
+                "    from [books]\n"
+                ")\n"
+                "select\n"
+                "    [original].*\n"
+                "from\n"
+                "    [original]\n"
+                "    join [books_fts] on [original].rowid = [books_fts].rowid\n"
+                "where\n"
+                "    [books_fts] match :query\n"
+                "order by\n"
+                "    [books_fts].rank"
+            ),
+        ),
+        (
+            {"columns": ["title"], "order_by": "rowid", "limit": 10},
+            "FTS5",
+            (
+                "with original as (\n"
+                "    select\n"
+                "        rowid,\n"
+                "        [title]\n"
+                "    from [books]\n"
+                ")\n"
+                "select\n"
+                "    [original].[title]\n"
+                "from\n"
+                "    [original]\n"
+                "    join [books_fts] on [original].rowid = [books_fts].rowid\n"
+                "where\n"
+                "    [books_fts] match :query\n"
+                "order by\n"
+                "    rowid\n"
+                "limit 10"
+            ),
+        ),
+        (
+            {"columns": ["title"]},
+            "FTS4",
+            (
+                "with original as (\n"
+                "    select\n"
+                "        rowid,\n"
+                "        [title]\n"
+                "    from [books]\n"
+                ")\n"
+                "select\n"
+                "    [original].[title]\n"
+                "from\n"
+                "    [original]\n"
+                "    join [books_fts] on [original].rowid = [books_fts].rowid\n"
+                "where\n"
+                "    [books_fts] match :query\n"
+                "order by\n"
+                "    rank_bm25(matchinfo([books_fts], 'pcnalx'))"
+            ),
+        ),
+    ],
+)
+def test_search_sql(kwargs, fts, expected):
+    db = Database(memory=True)
+    db["books"].insert(
+        {
+            "title": "Habits of Australian Marsupials",
+            "author": "Marlee Hawkins",
+        }
+    )
+    db["books"].enable_fts(["title", "author"], fts_version=fts)
+    sql = db["books"].search_sql(**kwargs)
+    assert sql == expected
